@@ -3,6 +3,7 @@ from .game import run, validate
 from .level import Level
 from .levels import levels  # This name overlap mostly hides the levels module which I prefer anyway
 from .terrains import terrains  # Ditto
-from .solution import Solution
+from .solution import Solution, Score
 from .molecule import Molecule
 from .elements import Element, elements, elements_dict  # Ditto
+from .exceptions import *
